{*******************************************************************}
{    DIB for TPColor.                                               }
{    All functions for work with DIB here are assembled.            }
{    Copyright (c) 1999-2001 Andrey Filatkin                        }
{    All Rights Reserved                                            }
{    anfilat@chat.ru ;   http://www.chat.ru/~anfilat                }
{    Last modified - 6 february 2001                                }
{*******************************************************************}
unit DIBWork;

interface

uses Windows;

type
  TLogPaletteDIB = record
    palVersion : Word;
    palNumEntries : Word;
    palEntry : array[0..255] of TPaletteEntry;
  end;

  TDIBWork = class
  private
    FBits      : Pointer;
    DWordWidth : Integer;
    BitmapInfo : PBitmapInfo;
    hDIB       : HBITMAP;
    FWidth  : Integer;
    FHeight : Integer;
    function GetPixel(X, Y: Integer): Integer;
    procedure SetPixel(X, Y: Integer; Value: Integer);
  public
    { Creates an instance of TDIBWork. Call Create to instantiate
    a TDIBWork object. Create allocates memory for the instance
    of TDIBWork and establishes a current palette. AWidth and AHeight
    - Width and Height bitmap.}
    constructor Create(AWidth, AHeight : Integer);
    { Destroys an instance of TDIBWork. Destroy frees the memory for
    the TDIBWork object.}
    destructor Destroy; override;
    { Use for clear DIB. Fill fon color.}
    procedure Clear;
    {Renders the DIB image of a control. }
    procedure Paint(destDC : hDC; ALeft,ATop : Integer); virtual;
    {Use for change size of DIB.}
    procedure Resize(Width,Height : Integer);
    { Set new palette.}
    procedure SetPalette(LogPal : TLogPaletteDIB);
    {Pointer on bitmap pixels.}
    property Bits : Pointer read FBits;
    property WordWidth : Integer read DWordWidth;
    {Specifies the color of the pixels within the DIB. Value Pixels
    is pointer on logical palette entry.
    Read Pixels to learn the color at a specific pixel position.
    Write Pixels to change the color of individual pixels on the bitmap.}
    property Pixels[X,Y : integer] : Integer read GetPixel write SetPixel;
    { Specifies the vertical size of the bitmap in pixels.
    For change the height use Resize.}
    property Height : Integer read FHeight;
    { Specifies the Horizontal size of the bitmap in pixels.
    For change the width use Resize.}
    property Width : Integer read FWidth;
  end;

implementation

uses SysUtils;

{ TDIBWork }

constructor TDIBWork.Create;
begin
  inherited Create;
  GetMem(BitmapInfo, sizeof(TBitmapInfo) + 256 * sizeof(TRGBQuad));
  with BitmapInfo.bmiHeader do
  begin
    biSize          := SizeOf(TBITMAPINFOHEADER);
    biPlanes        := 1;
    biBitCount      := 8;
    biCompression   := BI_RGB;
    biSizeImage     := 0;
    biXPelsPerMeter := 0;
    biYPelsPerMeter := 0;
    biClrUsed       := 0;
    biClrImportant  := 0;
  end;
  Resize(AWidth, AHeight);
end;

destructor TDIBWork.Destroy;
begin
  FreeMem(BitmapInfo);
  if hDIB <> 0 then DeleteObject(hDIB);
  inherited Destroy;
end;

procedure TDIBWork.SetPalette(LogPal : TLogPaletteDIB);
var
  ScreenDC, DC : HDC;
  OldBitmap : HBITMAP;
  RGBQuads : array[0..255] of TRGBQuad;
  i : Integer;
begin
  ScreenDC := GetDC(0);
  DC := CreateCompatibleDC(ScreenDC);
  try
    OldBitmap := SelectObject(DC, hDIB);
    for i := 0 to 255 do
    begin
      RGBQuads[i].rgbRed      := LogPal.palEntry[i].peRed;
      RGBQuads[i].rgbGreen    := LogPal.palEntry[i].peGreen;
      RGBQuads[i].rgbBlue     := LogPal.palEntry[i].peBlue;
      RGBQuads[i].rgbReserved := 0;
    end;
    SetDIBColorTable(DC,0,256,RGBQuads);
    SelectObject(DC, OldBitmap);
  finally
    DeleteDC(DC);
    ReleaseDC(0, ScreenDC);
  end;
end;

procedure TDIBWork.SetPixel(X, Y: Integer; Value: Integer);
begin
//  if (x < FWidth) and (x>=0) and (y<FHeight) and (y>=0) then
     pByte(Integer(FBits) + y * DWordWidth + x)^ := Value
end;

function  TDIBWork.GetPixel(X, Y: Integer): Integer;
begin
//  if (x < FWidth) and (x>=0) and (y<FHeight) and (y>=0) then
    Result := pByte(Integer(FBits) + y * DWordWidth + x)^
//  else
//     Result :=0;
end;

procedure TDIBWork.Paint(destDC : hDC; ALeft,ATop : Integer);
var
  ScreenDC, DC : HDC;
  OldBitmap : HBITMAP;
begin
  ScreenDC := GetDC(0);
  DC := CreateCompatibleDC(ScreenDC);
  try
    OldBitmap := SelectObject(DC, hDIB);
    BitBlt(destDC, ALeft, ATop, FWidth, FHeight, DC, 0, 0, SRCCOPY);
    SelectObject(DC, OldBitmap);
  finally
    DeleteDC(DC);
    ReleaseDC(0, ScreenDC);
  end;
end;

procedure TDIBWork.Resize(Width, Height : Integer);
var
  ScreenDC, DC : HDC;
  err : Integer;
begin
  ScreenDC := GetDC(0);
  DC := CreateCompatibleDC(ScreenDC);
  try
    DWordWidth := ((Width + 3) shr 2) shl 2;
    with BitmapInfo.bmiHeader do
    begin
      biWidth  := Width;
      biHeight := Height;
    end;
    FWidth := BitmapInfo.bmiHeader.biWidth;
    FHeight := BitmapInfo.bmiHeader.biHeight;
    if hDIB <> 0 then DeleteObject(hDIB);
    hDIB := 0;
    FBits := nil;
    if (FWidth<>0) and (FHeight<>0) then
    begin
      hDIB := CreateDIBSection(DC, BitmapInfo^, DIB_PAL_COLORS, FBits, 0, 0);
      if FBits = nil then
      begin
        err := GetLastError;
        if err<>0 then raise Exception.Create(SysErrorMessage(err))
        else raise Exception.Create('Error create DIB');
      end;
    end;
  finally
    DeleteDC(DC);
    ReleaseDC(0, ScreenDC);
  end;
end;

procedure TDIBWork.Clear;
begin
  FillMemory(pByte(Integer(FBits)), FHeight * DWordWidth, 0);
end;

{ End TDIBWork }

end.
